@@ -12,16 +12,13 @@ NumericVector get_hasse_impl(DataFrame scores, List serial_pref) {
   int ntuples = col1.size();
   
   // De-Serialize preference
-  pref* p = CreatePreference(serial_pref, scores);
+  ppref p = CreatePreference(serial_pref, scores);
 
   // Get edgelist (concatenated, to be transformed to a matrix afterwards)
-  std::list<int> edges = get_transitive_reduction(p, ntuples);
-  
-  // Delete preference
-  delete p;
+  list<int> edges = get_transitive_reduction(p, ntuples);
     
   NumericMatrix res(2, edges.size()/2);
-  std::copy(edges.begin(), edges.end(), res.begin());
+  copy(edges.begin(), edges.end(), res.begin());
   
   return(res);
 }
@@ -35,10 +32,10 @@ NumericVector get_hasse_impl(DataFrame scores, List serial_pref) {
 
 // Return transitive reduction as 1-dim list (x1,x2,x3,x4) means
 // x1 < x2 and x3 < x4 in the sense of the transitive reduction
-std::list<int> get_transitive_reduction(pref* p, int& ntuples) {
+list<int> get_transitive_reduction(ppref p, int& ntuples) {
 
   // The edgelist
-	std::list<int> edges;
+	list<int> edges;
 
 	// Naive approach for transitive reduction: Check all pairs and check if there exists some element between them
 	for (int i = 0; i < ntuples; i++)
